@@ -515,7 +515,11 @@ int ret;
 		if ( ret )
 			return ret;
 	}
-	return 0;
+	if ( b1 == 0 && b2 == 0 )
+		return 0;
+	if ( b1 == 0 )
+		return -1;
+	return 1;
 }
 
 L_CHAR *
@@ -1014,6 +1018,8 @@ RESOURCE * r;
 int rid;
 char * e_param;
 XL_SEXP * ret;
+
+
 	rid = 0;
 	for ( ; sf ; sf = sf->next ) {
 		if ( l_strcmp(sf->name,l_string(std_cm,"rid")) == 0 ) {
