@@ -358,17 +358,20 @@ LOAD_RESOURCE_WORK * wp;
 		r->h.flags &= ~RF_LOAD_EXTEND_PROC;
 		wakeup_task((int)r);
 	}
-	if ( w->wup_func )
-		(*w->wup_func)();
 	wakeup_task((int)w->wup);
 }
 
 void
 res_end_op_extend(LOAD_RESOURCE_WORK * w,int err,RESOURCE * r)
 {
+int (* func)();
+
 	lock_task(res_lock);
+	func = w->wup_func;
 	_res_end_op_extend(w,err,r);
 	unlock_task(res_lock,"res_end_op_extend");
+	if ( func )
+		(*func)();
 }
 
 void
@@ -915,9 +918,6 @@ RESOURCE_TYPE_LIST * rt;
 		wp1 = w;
 		w = w->que_next;
 		wp1->que_next = 0;
-ss_printf("EXTEND INSERT %ls %i %x\n",
-get_url_str2(&wp1->result->h.entry),
-wp1->err,wp1->result->h.flags);
 
 		if ( wp1->err != LRWE_HANDLING )
 			goto free_next;
@@ -926,9 +926,6 @@ wp1->err,wp1->result->h.flags);
 			er_panic("_lr_extend_insert");
 		if ( wp1->result->h.flags & 
 				(RF_LOAD_EXTEND|RF_LOAD_EXTEND_PROC) ){
-ss_printf("EE %ls %x\n",
-get_url_str2(&wp1->result->h.entry),
-wp1->result->h.flags);
 			goto free_next_in_lock;
 		}
 		wp1->result->h.flags |= RF_LOAD_EXTEND_PROC;
@@ -937,7 +934,6 @@ wp1->result->h.flags);
 
 		rt = get_resource_type_by_no(0,wp1->result->h.type);
 		if ( rt->loading_function ) {
-ss_printf("EE 1\n");
 			(*rt->loading_function)(wp1);
 			wp1->rt = rt;
 
@@ -945,14 +941,12 @@ ss_printf("EE 1\n");
 			lp = wp1;
 		}
 		else {
-ss_printf("EE 2\n");
 			res_end_op_extend(wp1,LRWE_OK,wp1->result);
 			goto free_next;
 		}
 		continue;
 
 	free_next_in_lock:
-ss_printf("E 5\n");
 		free_lrw_que_next(wp1);
 		unlock_task(res_lock,"_lr_option_insert");
 		continue;
@@ -961,7 +955,6 @@ ss_printf("E 5\n");
 	}
 	if ( lp )
 		_load_meta_list_insert(ses,&res_lp1,"data",0,lp,Q_WAIT_WAIT);
-ss_printf("EX END\n");
 }
 
 
@@ -1105,14 +1098,11 @@ int i;
 
 		lock_task(res_lock);
 		unlock_task(res_lock,"_load_resource_task1");
-ss_printf("EXTEND ++ %i\n",que->total_cnt);
 		wp = delete_res_queue_handle_max(que,key);
 		if ( wp == 0 ) {
 			gc_pop(0,0);
 			break;
 		}
-ss_printf("EXTEND + %i\n",que->total_cnt);
-ss_printf("EXTEND 1 %ls %x\n",get_url_str2(&wp->result->h.entry),wp->que_next);
 		_lr_extend_insert(ses,wp,key);
 
 		gc_pop(0,0);
@@ -1153,14 +1143,12 @@ int ret_f;
 	for ( ; ; ) {
 		gc_push(0,0,"_load_resource_task3");
 		w = lp = __load_meta_list_delete(&ret_f,ses,&res_lp1,key,1);
-if (lp)
-ss_printf("EXTEND 2 %ls\n",get_url_str2(&lp->result->h.entry));
+
 		if ( ret_f == 0 ) {
 			gc_pop(0,0);
 			break;
 		}
 		for ( ; lp  ; lp = lp->que_next ) {
-ss_printf("EXTEND 2 -- %ls\n",get_url_str2(&lp->result->h.entry));
 			if ( lp->rt->setup_function )
 				(*lp->rt->setup_function)(lp);
 			res_end_op_extend(lp,LRWE_OK,lp->result);
@@ -1526,8 +1514,6 @@ lr_extend(RESOURCE * r,int (*wup_func)())
 {
 LOAD_RESOURCE_WORK * wp;
 
-ss_printf("/////////////////////////////////////////////\n");
-ss_printf("lr_EXTEND INS %ls %x\n",get_url_str2(&r->h.entry),r->h.flags);
 	wp = new_lrw(0);
 	wp->err = LRWE_HANDLING;
 	wp->wup_func = wup_func;
@@ -1535,7 +1521,6 @@ ss_printf("lr_EXTEND INS %ls %x\n",get_url_str2(&r->h.entry),r->h.flags);
 	wp->h.key = get_server_key(&r->h.entry);
 	wp->que_pri = 0;
 	insert_queue(&res_extend,wp,1);
-ss_printf("lr_EXTEND QUE %i\n",res_extend.total_cnt);
 	return 0;
 }
 
