@@ -26,8 +26,10 @@
 #include	"resource.h"
 #include	"pri_level.h"
 
-MP_ROUTE *	route_cache;
-MP_ROUTE *	free_list;
+
+MP_CACHE direct;
+MP_CACHE indirect;
+
 MP_ERR_ROUTE *	err_route_cache;
 
 SEM route_lock;
@@ -48,6 +50,23 @@ SYS_QUEUE	rr_que1,rr_que2;
 
 MP_ROUTE_T * wait_list;
 
+void
+init_mp_cache(MP_CACHE * c)
+{
+int i;
+MP_ROUTE * r;
+	c->cache = d_alloc(sizeof(MP_ROUTE)*MP_ROUTE_CACHE_SIZE,22);
+	memset(c->cache,0,sizeof(MP_ROUTE)*MP_ROUTE_CACHE_SIZE);
+	c->root = &c->cache[0];
+	c->root->next = c->root->prev = c->root;
+	c->free_list = 0;
+	for ( i = 1 ; i < MP_ROUTE_CACHE_SIZE ; i ++ ) {
+		r = &c->cache[i];
+		r->next = c->free_list;
+		c->free_list = r;
+	}
+}
+
 void
 init_routing()
 {
@@ -61,18 +80,8 @@ void gc_routing_get();
 void gc_gb_sexp();
 
 	route_lock = new_lock(LL_ROUTE_LOCK);
-	route_cache = d_alloc(sizeof(MP_ROUTE)*MP_ROUTE_CACHE_SIZE,123);
-	r = &route_cache[0];
-	r->next = r->prev = r;
-	for ( i = 1 ; i < MP_ROUTE_CACHE_SIZE ; i ++ ) {
-		r = &route_cache[i];
-		r->next = free_list;
-		free_list = r;
-		r->prev = 0;
-		r->head = r->tail = 0;
-		r->lock = 0;
-		r->lwait = 0;
-	}
+	init_mp_cache(&direct);
+	init_mp_cache(&indirect);
 
 	memset(&rr_que1,0,sizeof(SYS_QUEUE));
 	rr_que1.flags = QF_STACK;
@@ -110,10 +119,10 @@ gc_routing(MP_ROUTE_T * n)
 }
 
 void
-insert_rring(MP_ROUTE * r)
+insert_rring(MP_CACHE * c,MP_ROUTE * r)
 {
-	r->prev = route_cache;
-	r->next = route_cache->next;
+	r->prev = c->root;
+	r->next = c->root->next;
 	r->prev->next = r;
 	r->next->prev = r;
 }
@@ -309,41 +318,119 @@ MP_PATH * p1,  * p2;
 }
 
 void
-_delete_route(MP_ROUTE * r)
+_delete_route(MP_CACHE * c,MP_ROUTE * r)
 {
+	if ( &c->cache[0] >= r )
+		er_panic("_Delete_route(!)");
+	if ( &c->cache[MP_ROUTE_CACHE_SIZE] <= r )
+		er_panic("_delete_route(2)");
 	delete_rring(r);
 	free_path(r);
-	r->next = free_list;
-	free_list = r;
+	r->next = c->free_list;
+	c->free_list = r;
 }
 
+
+void
+free_mp_path(MP_PATH * p)
+{
+MP_PATH * p2;
+	for ( ; p ; ) {
+		p2 = p->next;
+		d_f_ree(p->url);
+		d_f_ree(p);
+		p = p2;
+	}
+}
+
+void
+free_copy_route(MP_ROUTE * r)
+{
+	free_mp_path(r->head);
+	d_f_ree(r);
+}
+
+
+
 MP_ROUTE *
-_new_route()
+_copy_route(MP_ROUTE * r)
+{
+MP_PATH * from, * to;
+MP_ROUTE * ret;
+	ret = d_alloc(sizeof(*ret),22);
+	*ret = *r;
+	ret->head = ret->tail = 0;
+
+	for ( from = r->head ; from ; from = from->next ) {
+		to = d_alloc(sizeof(*to),3);
+		*to = *from;
+		to->url = ll_copy_str(from->url,22);
+		to->next = 0;
+		if ( ret->head ) {
+			ret->tail->next = to;
+			ret->tail = to;
+		}
+		else 	ret->head = ret->tail = to;
+	}
+
+	ret->next = ret->prev = 0;
+	ret->sts = RS_COPY;
+	ret->lock = 1;
+	ret->lwait = 0;
+	return ret;
+}
+
+
+
+MP_PATH *
+copy_mp_path(MP_PATH * p)
+{
+MP_PATH * ret;
+	ret = d_alloc(sizeof(*ret),22);
+	ret->next = 0;
+	ret->url = ll_copy_str(p->url,3);
+	return ret;
+}
+
+
+MP_PATH *
+sr_url(MP_PATH * path,L_CHAR * u)
+{
+MP_PATH * ret;
+	for ( ret = path ; ret ; ret = ret->next )
+		if ( l_strcmp(ret->url,u) == 0 )
+			return ret;
+	return 0;
+}
+
+
+MP_ROUTE *
+_new_route(MP_CACHE * cc)
 {
 MP_ROUTE * r;
-	if ( free_list ) {
-		r = free_list;
-		free_list = r->next;
-		insert_rring(r);
+	if ( cc->free_list ) {
+		r = cc->free_list;
+		cc->free_list = r->next;
+		insert_rring(cc,r);
 		return r;
 	}
 	else {
-		r = route_cache->prev;
-		for ( ; r != route_cache && r->lock ; r = r->prev );
-		if ( r == route_cache )
+		r = cc->root->prev;
+		for ( ; r != cc->root && r->lock ; r = r->prev );
+		if ( r == cc->root )
 			er_panic("_new_route");
 		free_path(r);
-		insert_rring(r);
+		insert_rring(cc,r);
 		return r;
 	}
 }
 
 MP_ROUTE *
-_new_route2(L_CHAR * start,L_CHAR * target)
+_new_route2(MP_CACHE * cc,L_CHAR * start,L_CHAR * target)
 {
 MP_ROUTE * r;
 MP_PATH * p1;
-	r = _new_route();
+	r = _new_route(cc);
 	r->head = p1 = d_alloc(sizeof(*p1),346);
 	p1->url = ll_copy_str(start,235);
 	p1->next = 0;
@@ -356,7 +443,7 @@ MP_PATH * p1;
 }
 
 MP_ROUTE *
-_search_route(L_CHAR * start,L_CHAR * target)
+_search_route(MP_CACHE * cc,L_CHAR * start,L_CHAR * target)
 {
 MP_ROUTE * inner;
 MP_ROUTE * r;
@@ -364,13 +451,13 @@ MP_PATH * p, * p_start, * p_target, * p1, * p2;
 int f;
 	inner = 0;
 	f = 0;
-	for ( r = route_cache->next ; r != route_cache ; r = r->next ) {
+	for ( r = cc->root->next ; r != cc->root ; r = r->next ) {
 		if ( r->head == 0 )
 			continue;
 		if ( l_strcmp(r->head->url,start) == 0 &&
 				l_strcmp(r->tail->url,target) == 0 ) {
 			delete_rring(r);
-			insert_rring(r);
+			insert_rring(cc,r);
 			return r;
 		}
 		if ( inner )
@@ -406,7 +493,7 @@ int f;
 	}
 	if ( inner == 0 )
 		return 0;
-	r = _new_route();
+	r = _new_route(cc);
 	if ( f == 2 ) {
 		r->head = r->tail = 0;
 		for ( p1 = p_start ; ; p1 = p1->next ) {
@@ -441,6 +528,33 @@ int f;
 }
 
 
+void
+_delete_route_cache(MP_CACHE * cc,L_CHAR * start,L_CHAR * target)
+{
+int cmp;
+L_CHAR * tmp;
+MP_ROUTE * mp;
+	switch ( l_strcmp(start,target) ) {
+	case -1:
+		break;
+	case 0:
+		return;
+	case 1:
+		tmp = start;
+		start = target;
+		target = tmp;
+	}
+	for ( mp = cc->root->next ; mp != cc->root ; mp = mp->next ) {
+		if ( mp->lock )
+			continue;
+		if ( l_strcmp(mp->head->url,start) == 0 &&
+				l_strcmp(mp->tail->url,target) == 0 ) {
+			_delete_route(cc,mp);
+			return;
+		}
+	}
+}
+
 MP_ROUTE *
 _insert_route(MP_ROUTE * r,XL_SEXP * list,L_CHAR * start,L_CHAR * target)
 {
@@ -495,6 +609,208 @@ err:
 }
 
 
+MP_PATH *
+path_optimization(MP_PATH * path)
+{
+MP_PATH * p1, * p2, * p3;
+	for ( p1 = path ; p1 ; p1 = p1->next ) {
+		for ( p2 = p1->next ; p2 ; p2 = p2->next )
+			if ( l_strcmp(p1->url,p2->url) == 0 )
+				goto opt;
+		continue;
+	opt:
+		p3 = p1->next;
+		p1->next = p2->next;
+		p2->next = 0;
+		free_mp_path(p3);
+	}
+	return path;
+}
+
+MP_PATH *
+_resolve_indirect_path(L_CHAR * start,L_CHAR * target)
+{
+MP_ROUTE * st, * tar;
+MP_PATH * p_st,* p_tar;
+MP_PATH * start_ptr, * target_ptr;
+MP_PATH * p_st_result, * p, *pp, ** p_ptr;
+MP_PATH * p_tar_result;
+	for  ( st = direct.root->next ; st != direct.root ; st = st->next ) {
+		if ( st->sts != RS_OK )
+			continue;
+		start_ptr = sr_url(st->head,start);
+		if ( start_ptr == 0 )
+			continue;
+		for ( tar = direct.root->next ; tar != direct.root ;
+				tar = tar->next ) {
+			if ( tar->sts != RS_OK )
+				continue;
+			target_ptr = sr_url(tar->head,target);
+			if ( target_ptr == 0 )
+				continue;
+			for ( p_tar = tar->head ; p_tar ;
+					p_tar = p_tar->next ) {
+				p_st = sr_url(st->head,p_tar->url);
+				if ( p_st )
+					goto ok;
+			}
+		}
+	}
+	return 0;
+ok:
+	p_st_result = 0;
+	for ( p = p_st ; p ; p = p->next ) {
+		pp = copy_mp_path(p);
+		pp->next = p_st_result;
+		p_st_result = pp;
+		if ( p == start_ptr )
+			break;
+	}
+	if ( p == 0 ) {
+		free_mp_path(p_st_result);
+		p_st_result = 0;
+		p_ptr = &p_st_result;
+		for ( p = start_ptr ; ; p = p->next ) {
+			pp = copy_mp_path(p);
+			*p_ptr = pp;
+			p_ptr = &pp->next;
+			if ( p == p_st )
+				break;
+		}
+	}
+	p_tar_result = 0;
+	for ( p = target_ptr ; p && p != p_tar ; p = p->next ) {
+		pp = copy_mp_path(p);
+		pp->next = p_tar_result;
+	}
+	if ( p == 0 ) {
+		free_mp_path(p_tar_result);
+		p_tar_result = 0;
+		p_ptr = &p_tar_result;
+		for ( p = p_tar->next ; ; p = p->next ) {
+			pp = copy_mp_path(p);
+			*p_ptr = pp;
+			p_ptr = &pp->next;
+			if ( p == target_ptr )
+				break;
+		}
+	}
+	for ( p = p_st_result ; p->next ; p = p->next );
+	p->next = p_tar_result;
+	return path_optimization(p_st_result);
+}
+
+MP_ROUTE *
+_resolve_direct(L_CHAR * start,L_CHAR * target)
+{
+int cmp;
+MP_ROUTE *ret;
+	cmp = l_strcmp(start,target);
+	switch ( cmp ) {
+	case -1:
+		ret = _search_route(&direct,start,target);
+		break;
+	case 0:
+		return 0;
+	case 1:
+		ret = _search_route(&direct,target,start);
+		break;
+	default:
+		er_panic("resolve_direct");
+	}
+	if ( ret == 0 )
+		return MP_NO_ROUTE;
+	ret->lock ++;
+	return ret;
+}
+
+MP_ROUTE *
+_resolve_indirect(int * sts,L_CHAR * start,L_CHAR * target)
+{
+int cmp;
+MP_ROUTE * ret;
+MP_PATH * path, * p;
+int _sts;
+	cmp = l_strcmp(start,target);
+	switch ( cmp ) {
+	case -1:
+		_sts = RI_UNRESOLVE;
+		ret = _search_route(&direct,start,target);
+		if ( ret ) {
+			switch ( ret->sts ) {
+			case RS_OK:
+				*sts = RI_DIRECT;
+				ret->lock ++;
+				return ret;
+			case RS_ERR:
+				_sts = RI_ERR;
+				break;
+			case RS_LOADING:
+				_sts = RI_LOADING;
+				break;
+			default:
+				_sts = RI_ERR;
+			}
+		}
+		ret = _search_route(&indirect,start,target);
+		if ( ret ) {
+			*sts = RI_INDIRECT;
+			ret->lock ++;
+			return ret;
+		}
+		path = _resolve_indirect_path(start,target);
+		if ( path == 0 ) {
+			*sts = _sts;
+			return MP_NO_ROUTE;
+		}
+		break;
+	case 0:
+		*sts = RI_DIRECT;
+		return 0;
+	case 1:
+		_sts = RI_UNRESOLVE;
+		ret = _search_route(&direct,target,start);
+		if ( ret ) {
+			switch ( ret->sts ) {
+			case RS_OK:
+				*sts = RI_DIRECT;
+				ret->lock ++;
+				return ret;
+			case RS_ERR:
+				_sts = RI_ERR;
+				break;
+			case RS_LOADING:
+				_sts = RI_LOADING;
+				break;
+			default:
+				_sts = RI_ERR;
+			}
+		}
+		ret = _search_route(&indirect,target,start);
+		if ( ret ) {
+			*sts = RI_INDIRECT;
+			ret->lock ++;
+			return ret;
+		}
+		path = _resolve_indirect_path(target,start);
+		if ( path == 0 ) {
+			*sts = _sts;
+			return MP_NO_ROUTE;
+		}
+		break;
+	}
+	ret = _new_route(&indirect);
+	ret->head = path;
+	for ( p = path ; p->next ; p = p->next );
+	ret->tail = p;
+	ret->lock = 1;
+	ret->lwait = 0;
+	ret->sts = RS_OK;
+	*sts = RI_INDIRECT;
+	ret->lock ++;
+	return ret;
+}
+
 RRR_TYPE *
 _rrr_search(URL * u)
 {
@@ -618,7 +934,7 @@ ss_printf("RESOLVE_ERROR %ls %ls\n",n->start,n->target);
 		n->handle1_start = n->start;
 		n->handle1_target = n->target;
 		lock_task(route_lock);
-		n->route = _search_route(n->start,n->target);
+		n->route = _search_route(&direct,n->start,n->target);
 		if ( n->route ) {
 			n->route->lock ++;
 			n->flags |= RQT_ROUTE_OK;
@@ -629,7 +945,8 @@ ss_printf("RESOLVE_ERROR %ls %ls\n",n->start,n->target);
 		n->handle1_start = n->target;
 		n->handle1_target = n->start;
 		lock_task(route_lock);
-		n->route = _search_route(n->handle1_start,n->handle1_target);
+		n->route = _search_route(&direct,
+				n->handle1_start,n->handle1_target);
 		if ( n->route ) {
 			n->route->lock ++;
 			n->flags |= RQT_ROUTE_OK;
@@ -695,7 +1012,9 @@ URL u;
 		lock_task(route_lock);
 		if ( n->route == 0 ) {
 			n->route = 
-				_new_route2(n->handle1_start,
+				_new_route2(
+					&direct,
+					n->handle1_start,
 					n->handle1_target);
 			n->route->lock = 1;
 			wakeup_task((int)n);
@@ -803,6 +1122,9 @@ int flags;
 					n->handle1_start,
 					n->handle1_target) == 0 )
 				goto err;
+			_delete_route_cache(&indirect,
+				n->handle1_start,
+				n->handle1_target);
 			flags = n->flags;
 			n->flags |= RQT_ROUTE_OK;
 			wakeup_task((int)n->route);
@@ -816,9 +1138,10 @@ int flags;
 		if ( n->flags & RQT_ROUTE_REVERSE ){
 			_new_err_route(n->handle1_start,n->handle1_target);
 
-
+ss_printf("ERR %ls %ls\n",n->handle1_start,n->handle1_target);
 			goto err;
 		}
+ss_printf("ERR RETRY %ls %ls\n",n->handle1_start,n->handle1_target);
 		n->flags |= RQT_ROUTE_REVERSE;
 		unlock_task(route_lock,"rr_task2");
 
@@ -848,7 +1171,7 @@ int flags;
 			sleep_task((int)r,route_lock);
 			lock_task(route_lock);
 		}
-		_delete_route(r);
+		_delete_route(&direct,r);
 		gc_pop(0,0);
 
 	last:
@@ -871,17 +1194,67 @@ resolve_route(L_CHAR * start,L_CHAR * target,int wait_flag)
 {
 MP_ROUTE_T * n;
 int type;
-MP_ROUTE * ret;
+MP_ROUTE * ret, * _ret;
+int sts;
 
-	if ( wait_flag )
+
+	ret = 0;
+	lock_task(route_lock);
+	switch ( wait_flag ) {
+	case RR_WM_NO_WAIT:
+	case RR_WM_DIRECT:
+		break;
+	case RR_WM_DIRECT_NO_WAIT:
+		ret = _resolve_direct(start,target);
+		if ( ret && ret != MP_NO_ROUTE ) {
+			switch ( ret->sts ) {
+			case RS_LOADING:
+				ret = MP_NO_ROUTE;
+				break;
+			case RS_OK:
+				break;
+			case RS_ERR:
+				wait_flag = RR_WM_INDIRECT_NO_WAIT;
+				goto indirect_pos;
+			default:
+				er_panic("resolve_route");
+			}
+		}
+		wait_flag = RR_WM_NO_WAIT;
+		break;
+	case RR_WM_INDIRECT:
+	case RR_WM_INDIRECT_NO_WAIT:
+	indirect_pos:
+		ret = _resolve_indirect(&sts,start,target);
+		switch ( sts ) {
+		case RI_INDIRECT:
+		case RI_DIRECT:
+		case RI_ERR:
+			wait_flag = RR_WM_NO_WAIT;
+			break;
+		case RI_LOADING:
+		case RI_UNRESOLVE:
+			if ( wait_flag == RR_WM_INDIRECT )
+				wait_flag = RR_WM_DIRECT;
+			else	wait_flag = RR_WM_NO_WAIT;
+			break;
+		default:
+			er_panic("resolve_route");
+		}
+		break;
+	default:
+		er_panic("resolve_route");
+	}
+	unlock_task(route_lock,"resolve_route");
+
+	if ( wait_flag != RR_WM_NO_WAIT )
 		type = RQT_LOCK|RQT_ROUTE;
 	else	type = RQT_ROUTE;
 	n = new_mp_route_t(start,target,type);
 	_resolve_route(n);
 
 	lock_task(route_lock);
-	if ( wait_flag == 0 ) {
-		ret = 0;
+	if ( wait_flag == RR_WM_NO_WAIT ) {
 		goto end;
 	}
 	_wait_mp_route(n);
@@ -893,8 +1266,16 @@ ss_printf("STS %x\n",n->flags);
 	else	ret = n->route;
 	free_mp_route_t(n);
 end:
+	if ( ret == MP_NO_ROUTE )
+		_ret = MP_NO_ROUTE;
+	else if ( ret ) {
+		_ret = _copy_route(ret);
+		_flush_route(ret);
+	}
+	else	_ret = 0;
 	unlock_task(route_lock,"resolve_route");
-	return ret;
+
+	return _ret;
 }
 
 void
@@ -902,10 +1283,14 @@ _flush_route(MP_ROUTE * r)
 {
 	if ( r == 0 )
 		return;
+	if ( r == MP_NO_ROUTE )
+		return;
 	if ( r->lock == 0 )
 		er_panic("flush_route");
 	if ( r->lock > 0 )
 		r->lock --;
+	if ( r->lock == 0 && r->sts == RS_COPY )
+		free_copy_route(r);
 }
 
 void
@@ -1160,14 +1545,14 @@ set_cpu_msg(2008);
 
 
 MAP_HISTORY *
-resolve_mappath(int ses,L_CHAR * org,L_CHAR * target)
+resolve_mappath(int ses,L_CHAR * org,L_CHAR * target,int wait_mode)
 {
 MP_ROUTE * mpr;
 URL u;
 MAP_HISTORY * mh;
 
 
-	mpr = resolve_route(org,target,1);
+	mpr = resolve_route(org,target,wait_mode);
 	if ( mpr == MP_NO_ROUTE )
 		return MP_MH_NO_ROUTE;
 	if ( mpr == 0 )
@@ -1188,6 +1573,7 @@ L_CHAR * b_start, * b_target;
 int cmp;
 MP_ROUTE * r;
 int ret;
+int sts;
 
 	if ( search_err_route(start,target) )
 		return CRT_ERROR;
@@ -1195,20 +1581,24 @@ int ret;
 	b_target = get_body(target);
 	if ( b_start && b_target && l_strcmp(b_start,b_target) )
 		return CRT_ERROR;
+/*
 	cmp = l_strcmp(start,target);
 	if ( cmp == 0 )
 		return CRT_EXIST;
+*/
 	lock_task(route_lock);
+/*
 	switch ( cmp ) {
 	case -1:
-		r = _search_route(start,target);
+		r = _search_route(&direct,start,target);
 		break;
 	case 1:
-		r = _search_route(target,start);
+		r = _search_route(&direct,target,start);
 		break;
 	}
-	if ( r == 0 )
+	if ( r == 0 ) {
 		ret = CRT_NOTHING;
+	}
 	else switch ( r->sts ) {
 	case RS_OK:
 		ret = CRT_EXIST;
@@ -1219,6 +1609,26 @@ int ret;
 	default:
 		ret = CRT_ERROR;
 	}
+*/
+	r = _resolve_indirect(&sts,start,target);
+	switch ( sts ) {
+	case RI_DIRECT:
+		ret = CRT_EXIST;
+		break;
+	case RI_INDIRECT:
+		ret = CRT_EXIST;
+		break;
+	case RI_LOADING:
+		ret = CRT_LOADING;
+		break;
+	case RI_UNRESOLVE:
+		ret = CRT_NOTHING;
+		break;
+	case RI_ERR:
+		ret = CRT_ERROR;
+		break;
+	}
+	_flush_route(r);
 	unlock_task(route_lock,"check_routing_table");
 	return ret;
 }
@@ -1238,7 +1648,7 @@ retry:
 	if ( ret != CRT_EXIST )
 		return ret;
 
-	mpr = resolve_route(org,target,1);
+	mpr = resolve_route(org,target,RR_WM_INDIRECT);
 	if ( mpr == MP_NO_ROUTE ) {
 		ret_cnt --;
 		if ( ret_cnt <= 0 )
