@@ -65,12 +65,11 @@ set_cpu_msg(3);
 */
 
 
-
 retry:
 
 /*
-ss_printf("****************** winflame %i %ls\n",get_tid(),
-       get_url_str2(&wf->draw->h.entry));
+ss_printf("****************** winflame %i %ls %x\n",get_tid(),
+       get_url_str2(&wf->draw->h.entry),wf->flags);
 */
 
 set_cpu_msg(4);
@@ -83,6 +82,11 @@ set_cpu_msg(4);
 		break;
 	case 2:
 		lock_task(env.wf_lock);
+set_cpu_msg(5);
+		if ( wf->flags & WFF_FREE ) {
+			unlock_task(env.wf_lock,"load_task");
+			goto end;
+		}
 		if ( _check_exit_flame() == 0 ) {
 			unlock_task(env.wf_lock,"load_task");
 			goto retry;
@@ -90,18 +94,30 @@ set_cpu_msg(4);
 		wid = wf->id;
 		wakeup_task((int)exit_task);
 		_win_unlock(wf);
+set_cpu_msg(6);
 		for ( ; _check_exit_flame() ; ) {
 			sleep_task((int)_wf_redraw_pri,env.wf_lock);
 			lock_task(env.wf_lock);
+			if ( _get_wf_ptr(wid) != wf ) {
+				unlock_task(env.wf_lock,"load_task");
+				goto start;
+			}
+			if ( wf->flags & WFF_FREE ) {
+				unlock_task(env.wf_lock,"load_task");
+				goto start;
+			}
 		}
+set_cpu_msg(7);
 		if ( _get_wf_ptr(wid) != wf ) {
 			unlock_task(env.wf_lock,"load_task");
 			goto start;
 		}
+set_cpu_msg(8);
 		if ( _win_lock(wf) < 0 ) {
 			unlock_task(env.wf_lock,"load_task");
 			goto start;
 		}
+set_cpu_msg(9);
 		unlock_task(env.wf_lock,"load_task");
 		goto retry;
 	default:
@@ -115,6 +131,7 @@ ss_printf("****************** spiral end %i %x\n",get_tid(),wf);
 end:
 set_cpu_msg(11);
 
+
 	win_unlock(wf);
 set_cpu_msg(12);
 
@@ -138,31 +155,42 @@ void _wf_redraw_pri();
 set_cpu_thr_type(13);
 
 	for ( ; ; ) {
+set_cpu_msg(1);
 		lock_task(env.wf_lock);
 	retry:
+set_cpu_msg(2);
 		fg = 0;
 		for ( wf = env.flame ; wf ; ) {
+set_cpu_msg(3);
 			wf2 = wf->next;
 			_win_wlock(wf);
 			_ex_lock();
 			if ( wf->flags & WFF_FREE ) {
 
 printf("EXIT TASK\n");
+set_cpu_msg(14);
 				exit_win_flame(wf);
 				wakeup_task((int)wf);
 				wakeup_task((int)exit_task);
 				wakeup_task((int)_wf_redraw_pri);
+if ( env.flame == 0 )
+printf("EXIT NO FLAME!!!\n");
+set_cpu_msg(5);
 				_ex_unlock();
 				fg = 1;
 			}
 			else {
+set_cpu_msg(6);
 				_ex_unlock();
 				_win_unlock(wf);
 			}
+set_cpu_msg(7);
 			wf = wf2;
 		}
+set_cpu_msg(8);
 		if ( fg )
 			goto retry;
+set_cpu_msg(9);
 		sleep_task((int)exit_task,env.wf_lock);
 	}
 }
@@ -189,10 +217,12 @@ set_cpu_thr_type(14);
 			unlock_task(env.wf_lock,"change_task");
 			ex_lock(); 
 
-			if ( (wf->flags&(WFF_POLY|WFF_PLOT|WFF_LUSTER))
-					== WFF_PLOT )
+			switch ( (wf->flags&(WFF_POLY|WFF_PLOT|WFF_LUSTER)) ) {
+			case 0:
+			case WFF_PLOT:
 				change_win_flame(wf,0,WFT_PLOT);
-			else	change_win_flame(wf,0,WFT_STD);
+				change_win_flame(wf,0,WFT_STD);
+			}
 			ex_unlock();
 			win_unlock(wf);
 		}
