@@ -38,20 +38,14 @@ int interface_flag;
 void _wf_flame_redraw();
 void _wf_set_redraw();
 
-void wf_null_handler();
-void wf_invoke_handler();
 
 void check_flame_task();
 
 void
 init_win_flame()
 {
-	null_handler = wf_null_handler;
-	invoke_handler = wf_invoke_handler;
-
 	env.wf_lock = new_lock(LL_WF);
 	env.seq = 1;
-
 /*
 	create_task(check_flame_task,0,0);
 */
@@ -76,6 +70,7 @@ WIN_FLAME * wf;
 			ss_printf("%i %x %i\n",wf->lock,wf->flags,wf->pri_adj);
 		}
 		unlock_task(env.wf_lock,"check_flame_task");
+		gv_cpu_direct();
 	}
 }
 
@@ -90,30 +85,6 @@ int ret;
 }
 
 
-void
-wf_null_handler()
-{
-WIN_FLAME * wf;
-	interface_flag = 0;
-	wakeup_task((int)&interface_flag);
-	if ( null_wf_spiral ) {
-		lock_task(env.wf_lock);
-set_t_msg(1);
-		for ( wf = env.flame ; wf ; wf = wf->next ) {
-			wakeup_task((int)&env.flame);
-		}
-		unlock_task(env.wf_lock,"wf_null_handler");
-set_t_msg(2);
-		null_wf_spiral = 0;
-	}
-}
-
-void
-wf_invoke_handler()
-{
-	interface_flag = 1;
-}
-
 int system_wfid;
 
 
@@ -156,7 +127,6 @@ retry:
 		id = wf->id;
 		sleep_task((int)wf,env.wf_lock);
 		lock_task(env.wf_lock);
-set_t_msg(3);
 		if ( _get_wf_ptr(id) == 0 )
 			return -1;
 		goto retry;
@@ -189,7 +159,6 @@ _win_unlock(WIN_FLAME * wf)
 WIN_FLAME * wf2;
 int _wf_redraw_pri();
 
-set_cpu_msg(7);
 	if ( wf->lock == 0 )
 		er_panic("_win_unlock");
 	if ( wf->lock > 0 )
@@ -199,7 +168,6 @@ set_cpu_msg(7);
 		wakeup_task((int)wf);
 		wakeup_task((int)_wf_redraw_pri);
 	}
-set_cpu_msg(8);
 }
 
 int
@@ -207,7 +175,6 @@ win_lock(WIN_FLAME * wf)
 {
 int ret;
 	lock_task(env.wf_lock);
-set_t_msg(5);
 	ret = _win_lock(wf);
 	unlock_task(env.wf_lock,"win_lock");
 	return ret;
@@ -801,8 +768,6 @@ WIN_FLAME * wf;
 void exit_task();
 	for ( ; ; ) {
 		lock_task(env.wf_lock);
-set_t_msg(17);
-printf("STABLE CHECK\n");
 		for ( wf = env.flame ; wf ; wf = wf->next ) {
 			if ( wf->flags & WFF_FREE )
 				break;
@@ -811,7 +776,6 @@ printf("STABLE CHECK\n");
 			unlock_task(env.wf_lock,"wf_wait_stable");
 			break;
 		}
-printf("STABLE WAIT\n");
 		sleep_task((int)exit_task,env.wf_lock);
 	}
 }
@@ -822,7 +786,7 @@ _free_win_flame(WIN_FLAME * wf)
 void exit_task();
 int _wf_redraw_pri();
 	wf->flags |= WFF_FREE;
-printf("FREE\n");
+ss_printf("FREE %ls\n",get_url_str2(&wf->draw->h.entry));
 	wakeup_task((int)exit_task);
 	wakeup_task((int)_wf_redraw_pri);
 }
@@ -2372,7 +2336,9 @@ set_t_msg(40);
 	return ret;
 }
 
-void
+
+
+int
 min_route_wf(int ses)
 {
 int cnt,i;
@@ -2381,7 +2347,9 @@ L_CHAR * target, * org, * t;
 MAP_HISTORY * mh;
 MP_ROUTE * mpr;
 URL u;
+int ret;
 
+	ret = 0;
 	cnt = 0;
 retry:
 	lock_task(env.wf_lock);
@@ -2390,7 +2358,7 @@ set_t_msg(41);
 	wf = _wf_get_current();
 	if ( wf == 0 ) {
 		unlock_task(env.wf_lock,"min_route_wf");
-		return;
+		return ret;
 	}
 	org = ll_copy_str(get_url_str2(&wf->draw->h.entry),1538);
 
@@ -2401,19 +2369,26 @@ set_t_msg(41);
 	}
 	if ( wf == 0 ) {
 		unlock_task(env.wf_lock,"min_route_wf");
-		return;
+		return ret;
 	}
 	target = ll_copy_str(get_url_str2(&wf->draw->h.entry),1539);
 	unlock_task(env.wf_lock,"min_route_wf");
 
-	mpr = resolve_route(org,target,1);
-	if ( mpr == 0 || mpr == MP_NO_ROUTE )
+	mpr = resolve_route(org,target,RR_WM_DIRECT_NO_WAIT);
+	if ( mpr == 0 ) {
 		goto flush_mpr2;
+	}
+	if ( mpr == MP_NO_ROUTE ) {
+		ret = -1;
+		goto flush_mpr2;
+	}
 	get_url2(&u,org,1569);
 	mh = get_mh_from_route(ses,&u,mpr);
 	free_url(&u);
-	if ( mh == MP_MH_NO_ROUTE )
+	if ( mh == MP_MH_NO_ROUTE ) {
+		ret = -1;
 		goto flush_mpr;
+	}
 
 	lock_task(env.wf_lock);
 set_t_msg(42);
@@ -2729,15 +2704,12 @@ printf("%x:%i",g_wf->flags,g_p);
 _ind_pri();
 */
 
-set_cpu_msg(1);
 	sleep_task((int)_wf_redraw_pri,env.wf_lock);
 	lock_task(env.wf_lock);
-set_t_msg(48);
 	if ( g_wf->aboat_count )
 		return -1;
 	if ( g_wf->flags & WFF_FREE )
 		return -1;
-set_cpu_msg(2);
 
 	goto retry;
 }
@@ -2998,3 +2970,22 @@ printf("CLICK 1\n");
 	return _wf_click(p,0);
 }
 
+unsigned int
+_wf_insert_time(WIN_FLAME * wf)
+{
+	return wf->insert_time;
+}
+
+unsigned int
+wf_insert_time(int wfid)
+{
+WIN_FLAME * wf;
+unsigned int ret;
+	lock_task(env.wf_lock);
+	wf = _get_wf_ptr(wfid);
+	if ( wf == 0 )
+		ret = 0;
+	else	ret = _wf_insert_time(wf);
+	unlock_task(env.wf_lock,"wf_insert_time");
+	return ret;
+}
