@@ -45,8 +45,8 @@ VERSION xl_code_version = {
 	"XL.code",
 	"XL",
 	"(C) 2000-2003 Hirohisa Mori",
-	"ver.A.01 (ASTROBOY)",
-	2003,5,27,
+	"ver.A.02.04 (ASTROBOY)",
+	2003,6,30,
 	0,
 	"version of XL code (src/xl)"
 };
