@@ -72,7 +72,9 @@ SEM		XoH_lock;
 unsigned int	cockie_cnt;
 unsigned int	cockie_start_time;
 STREAM_STOCK *	ss_list;
-int		ss_wait_tasks;
+int		ss_wait_tasks,ss_run_tasks;
+
+void ss_task();
 
 void
 check_fid_tick()
@@ -93,6 +95,8 @@ void empty_reply_task();
 */
 	create_task(empty_reply_task,0,1);
 
+	ss_run_tasks = 0;
+	create_task(ss_task,0,1);
 }
 
 
@@ -232,7 +236,6 @@ void
 _insert_ss(STREAM *s,unsigned int ip,XL_INTERPRETER * xli,PERMISSION_LIST * pl)
 {
 STREAM_STOCK * ss;
-void ss_task();
 
 	ss = d_alloc(sizeof(*ss),123);
 	ss->key.s = s;
@@ -243,8 +246,6 @@ void ss_task();
 	ss->next = ss_list;
 	ss_list = ss;
 
-	if ( ss_wait_tasks == 0 )
-		create_task(ss_task,0,1);
 	wakeup_task((int)&ss_list);
 }
 
@@ -377,13 +378,17 @@ HTTP_INFO o_info;
 		en_do(&ret_len,s_write,lst->con,&h,sizeof(h));
 
 		if ( e->flags & XoH_E_KEEP_ALIVE )
+{
+printf("insert_ss\n");
 			_insert_ss(lst->con,e->ip,e->xli,e->pl);
+}
 		else	s_close(lst->con);
 		free_HTTP_info(&lst->info);
 		d_f_ree(lst);
 	}
 	else {
 		e = _search_XoH_entry(lst->h.cockie);
+printf("search_xoh_entry %x\n",e);
 		if ( e == 0 ) {
 		XLoHTTP_HEADER h;
 
@@ -425,6 +430,10 @@ ss_task()
 STREAM * new;
 STREAM_STOCK * ss;
 int ret;
+printf("start ss_task\n");
+	lock_task(XoH_lock);
+	ss_run_tasks ++;
+	unlock_task(XoH_lock,"ss_task");
 	for ( ; ; ) {
 		lock_task(XoH_lock);
 		for ( ; ; ) {
@@ -435,26 +444,32 @@ int ret;
 				unlock_task(XoH_lock,"ss_task");
 				return;
 			}
+			ss_run_tasks --;
 			ss_wait_tasks ++;
 			sleep_task((int)&ss_list,XoH_lock);
+printf("*** ss wakeup ***\n");
 			lock_task(XoH_lock);
 			ss_wait_tasks --;
-			if ( ss_wait_tasks <= 0 )
+			ss_run_tasks ++;
+			if ( ss_wait_tasks <= 0 && ss_run_tasks < 10 )
 				create_task(ss_task,0,1);
 		}
 		unlock_task(XoH_lock,"ss_task");
 		ret = new_accept(&new,&ss->key,ss->xli,ss->pl);
 		switch ( ret ) {
 		case -1:
+printf("* error *\n");
 			/* error */
 			s_close(ss->key.s);
 			d_f_ree(ss);
 			break;
 		case 0:
+printf("* nothing *\n");
 			/* nothing to do */
 			d_f_ree(ss);
 			break;
 		case 1:
+printf("* launch *\n");
 			/* launch new stream */
 			launch_accept_thread(
 				new,
@@ -463,6 +478,8 @@ int ret;
 				ss->pl);
 			d_f_ree(ss);
 			break;
+		default:
+printf("*** panic !!! ***\n");
 		}
 	}
 }
@@ -538,6 +555,7 @@ int len;
 		for ( ; (lst = get_recv_seq_list(e)) == 0 &&
 				(e->flags & (XoH_E_EXIT|XoH_E_F_ERR))
 					 == 0 ; ) {
+printf("recv sleep...\n");
 			sleep_task((int)e,XoH_lock);
 			lock_task(XoH_lock);
 		}
